@@ -6,7 +6,9 @@
 #
 
 from .experiment import Experiment
+from .experimentcombinator import ExperimentCombinator
 from .repeatedexperiment import RepeatedExperiment
+from .summaryexperiment import SummaryExperiment
 
 from .lab import Lab
 from .clusterlab import ClusterLab
